@@ -5,38 +5,37 @@
 //      Minimal BOOTP functions
 //
 //==========================================================================
-//####ECOSGPLCOPYRIGHTBEGIN####
-// -------------------------------------------
-// This file is part of eCos, the Embedded Configurable Operating System.
-// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+// ####ECOSGPLCOPYRIGHTBEGIN####                                            
+// -------------------------------------------                              
+// This file is part of eCos, the Embedded Configurable Operating System.   
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 //
-// eCos is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free
-// Software Foundation; either version 2 or (at your option) any later version.
+// eCos is free software; you can redistribute it and/or modify it under    
+// the terms of the GNU General Public License as published by the Free     
+// Software Foundation; either version 2 or (at your option) any later      
+// version.                                                                 
 //
-// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
-// WARRANTY; without even the implied warranty of MERCHANTABILITY or
-// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-// for more details.
+// eCos is distributed in the hope that it will be useful, but WITHOUT      
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
+// for more details.                                                        
 //
-// You should have received a copy of the GNU General Public License along
-// with eCos; if not, write to the Free Software Foundation, Inc.,
-// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+// You should have received a copy of the GNU General Public License        
+// along with eCos; if not, write to the Free Software Foundation, Inc.,    
+// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
 //
-// As a special exception, if other files instantiate templates or use macros
-// or inline functions from this file, or you compile this file and link it
-// with other works to produce a work based on this file, this file does not
-// by itself cause the resulting work to be covered by the GNU General Public
-// License. However the source code for this file must still be made available
-// in accordance with section (3) of the GNU General Public License.
+// As a special exception, if other files instantiate templates or use      
+// macros or inline functions from this file, or you compile this file      
+// and link it with other works to produce a work based on this file,       
+// this file does not by itself cause the resulting work to be covered by   
+// the GNU General Public License. However the source code for this file    
+// must still be made available in accordance with section (3) of the GNU   
+// General Public License v2.                                               
 //
-// This exception does not invalidate any other reasons why a work based on
-// this file might be covered by the GNU General Public License.
-//
-// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
-// at http://sources.redhat.com/ecos/ecos-license/
-// -------------------------------------------
-//####ECOSGPLCOPYRIGHTEND####
+// This exception does not invalidate any other reasons why a work based    
+// on this file might be covered by the GNU General Public License.         
+// -------------------------------------------                              
+// ####ECOSGPLCOPYRIGHTEND####                                              
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -68,6 +67,9 @@
 #ifdef CYGINT_ISO_DNS
 #include <netdb.h>
 #endif
+#ifdef CYGPKG_NET_SNTP
+#include <cyg/sntp/sntp.h>
+#endif
 
 #ifndef CYGPKG_LIBC_STDIO
 #define perror(s) diag_printf(#s ": %s\n", strerror(errno))
@@ -83,12 +85,13 @@ do_bootp(const char *intf, struct bootp *recv)
     struct ifreq ifr;
     struct sockaddr_in cli_addr, serv_addr, bootp_server_addr;
     struct ecos_rtentry route;
-    int s, addrlen;
+    int s=-1;
+    socklen_t addrlen;
     int one = 1;
     struct bootp bootp_xmit;
     unsigned char mincookie[] = {99,130,83,99,255} ;
     struct timeval tv;
-    cyg_bool_t retcode = true;
+    cyg_bool_t retcode = false;
 
     // Ensure clean slate
     cyg_route_reinit();  // Force any existing routes to be forgotten
@@ -96,12 +99,12 @@ do_bootp(const char *intf, struct bootp *recv)
     s = socket(AF_INET, SOCK_DGRAM, 0);
     if (s < 0) {
         perror("socket");
-        return false;
+        goto out;
     }
 
     if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one))) {
         perror("setsockopt");
-        return false;
+        goto out;
     }
 
     addrp = (struct sockaddr_in *) &ifr.ifr_addr;
@@ -114,30 +117,30 @@ do_bootp(const char *intf, struct bootp *recv)
     strcpy(ifr.ifr_name, intf);
     if (ioctl(s, SIOCSIFADDR, &ifr)) {
         perror("SIOCSIFADDR");
-        return false;
+        goto out;
     }
 
     if (ioctl(s, SIOCSIFNETMASK, &ifr)) {
         perror("SIOCSIFNETMASK");
-        return false;
+        goto out;
     }
 
     /* the broadcast address is 255.255.255.255 */
     memset(&addrp->sin_addr, 255, sizeof(addrp->sin_addr));
     if (ioctl(s, SIOCSIFBRDADDR, &ifr)) {
         perror("SIOCSIFBRDADDR");
-        return false;
+        goto out;
     }
 
     ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
     if (ioctl(s, SIOCSIFFLAGS, &ifr)) {
         perror("SIOCSIFFLAGS");
-        return false;
+        goto out;
     }
 
     if (ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
         perror("SIOCGIFHWADDR");
-        return false;
+        goto out;
     }
 
     // Set up routing
@@ -159,7 +162,7 @@ do_bootp(const char *intf, struct bootp *recv)
     if (ioctl(s, SIOCADDRT, &route)) {
         if (errno != EEXIST) {
             perror("SIOCADDRT 3");
-            return false;
+            goto out;
         }
     }
 
@@ -170,15 +173,15 @@ do_bootp(const char *intf, struct bootp *recv)
     
     if(bind(s, (struct sockaddr *) &cli_addr, sizeof(cli_addr)) < 0) {
         perror("bind error");
-        return false;
+        goto out;
     }
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one))) {
         perror("setsockopt SO_REUSEADDR");
-        return false;
+        goto out;
     }
     if (setsockopt(s, SOL_SOCKET, SO_REUSEPORT, &one, sizeof(one))) {
         perror("setsockopt SO_REUSEPORT");
-        return false;
+        goto out;
     }
     
     memset((char *) &serv_addr, 0, sizeof(serv_addr));
@@ -190,7 +193,7 @@ do_bootp(const char *intf, struct bootp *recv)
     bzero(&bootp_xmit, sizeof(bootp_xmit));
     if (ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
         perror("SIOCGIFHWADDR");
-        return false;
+        goto out;
     }
     bootp_xmit.bp_htype = HTYPE_ETHERNET;
     bootp_xmit.bp_hlen = IFHWADDRLEN;
@@ -204,13 +207,16 @@ do_bootp(const char *intf, struct bootp *recv)
     if(sendto(s, &bootp_xmit, sizeof(struct bootp), 0, 
               (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
         perror("sendto error");
-        return false;
+        goto out;
     }
 
     tv.tv_sec = 5;
     tv.tv_usec = 0;
     setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
-
+    
+    // Everything passed here is a success.
+    retcode = true;
+    
     addrlen = sizeof(bootp_server_addr);
     if (recvfrom(s, recv, sizeof(struct bootp), 0,
                  (struct sockaddr *)&bootp_server_addr, &addrlen) < 0) {
@@ -236,11 +242,13 @@ do_bootp(const char *intf, struct bootp *recv)
     ifr.ifr_flags &= ~(IFF_UP | IFF_RUNNING);
     if (ioctl(s, SIOCSIFFLAGS, &ifr)) {
         perror("SIOCSIFFLAGS");
-        return false;
+        retcode = false;
     }
 
+ out:
     // All done with socket
-    close(s);
+    if (s != -1)
+      close(s);
     return retcode;
 }
 
@@ -255,7 +263,8 @@ void
 show_bootp(const char *intf, struct bootp *bp)
 {
     int i, len;
-    unsigned char *op, *ap = 0, optover;
+    unsigned char *op, optover;
+    char  *ap = 0;
     unsigned char name[128];
     struct in_addr addr[32];
     unsigned int length;
@@ -290,11 +299,14 @@ show_bootp(const char *intf, struct bootp *bp)
         op = &bp->bp_vend[4];
         while (*op != TAG_END) {
             switch (*op) {
+            case TAG_PAD:
+                op++;
+                continue;
             case TAG_SUBNET_MASK:
             case TAG_GATEWAY:
             case TAG_IP_BROADCAST:
             case TAG_DOMAIN_SERVER:
-                ap = (unsigned char *)&addr[0];
+                ap = (char *)&addr[0];
                 len = *(op+1);
                 for (i = 0;  i < len;  i++) {
                     *ap++ = *(op+i+2);
@@ -304,7 +316,7 @@ show_bootp(const char *intf, struct bootp *bp)
                 if (*op == TAG_IP_BROADCAST)  ap =  " IP broadcast";
                 if (*op == TAG_DOMAIN_SERVER) ap =  "domain server";
                 diag_printf("      %s: ", ap);
-                ap = (unsigned char *)&addr[0];
+                ap = (char *)&addr[0];
                 while (len > 0) {
                     diag_printf("%s", inet_ntoa(*(struct in_addr *)ap));
                     len -= sizeof(struct in_addr);
@@ -361,7 +373,15 @@ show_bootp(const char *intf, struct bootp *bp)
                         diag_printf(" %d",op[i]);
                 diag_printf("\n");
                 break;
-
+            case TAG_NTP_SERVER:
+              diag_printf("        NTP servers: ");
+              for ( i = 0 ; i < op[1]/4 ; i++) {
+                diag_printf("%d.%d.%d.%d ",
+                            op[1+i*4+1], op[1+i*4+2], 
+                            op[1+i*4+3], op[1+i*4+4]);
+              }
+              diag_printf("\n");
+              break;
             default:
                 diag_printf("Unknown option: %x/%d.%d:", *op, *op, *(op+1));
                 for ( i = 2; i < 2 + op[1]; i++ )
@@ -394,7 +414,11 @@ get_bootp_option(struct bootp *bp, unsigned char tag, void *opt,
             *length=max;                        \
             return true;                        \
         }                                       \
-        op += *(op+1)+2;                        \
+        if (*op == TAG_PAD) {                   \
+            op++;                               \
+        } else {                                \
+            op += *(op+1)+2;                    \
+        }                                       \
     }                                           \
 CYG_MACRO_END
 
@@ -408,10 +432,10 @@ CYG_MACRO_END
         return false;
 
     if ( 1 & optover ) // then the file field also holds options
-        SCANTAG( &bp->bp_file[0] );
+        SCANTAG( (unsigned char *)&bp->bp_file[0] );
 
     if ( 2 & optover ) // then the sname field also holds options
-        SCANTAG( &bp->bp_sname[0] );
+        SCANTAG( (unsigned char *)&bp->bp_sname[0] );
 
     return false;
 }
@@ -422,21 +446,22 @@ init_net(const char *intf, struct bootp *bp)
 {
     struct sockaddr_in *addrp;
     struct ifreq ifr;
-    int s;
+    int s=-1;
     int one = 1;
     struct ecos_rtentry route;
     struct in_addr netmask, gateway;
     unsigned int length;
-
+    int retcode = false;
+    
     s = socket(AF_INET, SOCK_DGRAM, 0);
     if (s < 0) {
         perror("socket");
-        return false;
+        goto out;
     }
 
     if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one))) {
         perror("setsockopt");
-        return false;
+        goto out;
     }
 
     addrp = (struct sockaddr_in *) &ifr.ifr_addr;
@@ -451,7 +476,7 @@ init_net(const char *intf, struct bootp *bp)
     strcpy(ifr.ifr_name, intf);
     if (ioctl(s, SIOCSIFADDR, &ifr)) {
         perror("SIOCIFADDR");
-        return false;
+        goto out;
     }
 
     length = sizeof(addrp->sin_addr);
@@ -459,14 +484,14 @@ init_net(const char *intf, struct bootp *bp)
         netmask = addrp->sin_addr;
         if (ioctl(s, SIOCSIFNETMASK, &ifr)) {
             perror("SIOCSIFNETMASK");
-            return false;
+            goto out;
         }
         // Must do this again so that [sub]netmask (and so default route)
         // is taken notice of.
         addrp->sin_addr = bp->bp_yiaddr;  // The address BOOTP gave us
         if (ioctl(s, SIOCSIFADDR, &ifr)) {
             perror("SIOCIFADDR 2");
-            return false;
+            goto out;
         }
     }
 
@@ -474,7 +499,7 @@ init_net(const char *intf, struct bootp *bp)
     if (get_bootp_option(bp, TAG_IP_BROADCAST, &addrp->sin_addr,&length)) {
         if (ioctl(s, SIOCSIFBRDADDR, &ifr)) {
             perror("SIOCSIFBRDADDR");
-            return false;
+            goto out;
         }
         // Do not re-set the IFADDR after this; doing *that* resets the
         // BRDADDR to the default!
@@ -483,7 +508,7 @@ init_net(const char *intf, struct bootp *bp)
     ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
     if (ioctl(s, SIOCSIFFLAGS, &ifr)) {
         perror("SIOCSIFFLAGS");
-        return false;
+        goto out;
     }
 
     // Set up routing
@@ -515,12 +540,13 @@ init_net(const char *intf, struct bootp *bp)
                   inet_ntoa(((struct sockaddr_in *)&route.rt_gateway)->sin_addr));
                 if (errno != EEXIST) {
                     perror("SIOCADDRT 3");
-                    return false;
+                    goto out;
                 }
             }
         }
     }
-    close(s);
+    retcode = true;
+    
 #ifdef CYGINT_ISO_DNS
     {
 #define MAX_IP_ADDR_LEN 16
@@ -536,7 +562,51 @@ init_net(const char *intf, struct bootp *bp)
         }
     }
 #endif
-    return true;
+
+#ifdef CYGNUM_NET_SNTP_UNICAST_MAXDHCP
+    {
+        struct in_addr dhcp_addrs[CYGNUM_NET_SNTP_UNICAST_MAXDHCP];
+
+        /* Removed any previously registered addresses */
+        cyg_sntp_set_servers(NULL, 0);
+
+        /* See if we received any NTP servers from DHCP */
+        length = sizeof(dhcp_addrs);
+        if (get_bootp_option(bp, TAG_NTP_SERVER, &dhcp_addrs[0], &length))
+        {
+        	static struct sockaddr ntp_servers[CYGNUM_NET_SNTP_UNICAST_MAXDHCP];
+            struct servent *service;
+        	cyg_uint32 num;
+
+            /* See how many addresses we got.  The length should always
+             * be a multiple of 4, but cut off any extra bytes and
+             * use what we got.
+             */
+            length /= sizeof(struct in_addr);
+
+            /* Fill out a sockaddr array for the NTP client */
+            service = getservbyname("ntp", "udp");
+            CYG_CHECK_DATA_PTR(service, "NTP service not found.");
+            memset(&ntp_servers[0], 0, sizeof(ntp_servers));
+            for (num = 0; num < length; num++)
+            {
+				struct sockaddr_in *saddr = (struct sockaddr_in *)&ntp_servers[num];
+
+                saddr->sin_len = sizeof(*saddr);
+                saddr->sin_family = AF_INET;
+                saddr->sin_port = service->s_port;  /* Already network-endian */
+                saddr->sin_addr = dhcp_addrs[num];  /* Already network-endian */
+            }
+
+            /* Configure the client with the array */
+            cyg_sntp_set_servers(&ntp_servers[0], num);
+        }
+    }
+#endif /* CYGNUM_NET_SNTP_UNICAST_MAXDHCP */
+ out:
+    if (s != -1) 
+      close(s);
+    return retcode;
 }
 
 #ifdef INET6
@@ -544,15 +614,16 @@ extern const struct in6_addr in6mask128;
 cyg_bool_t
 init_net_IPv6(const char *intf, struct bootp *bp, char *prefix)
 {
-    int s;
+    int s =-1 ;
     struct in6_aliasreq in6_addr;
     char in6_ip[128];
+    int retcode = false;
 
     // Set up non link-layer address
     s = socket(AF_INET6, SOCK_DGRAM, 0);
     if (s < 0) {
         perror("socket IPv6");
-        return false;
+        goto out;
     }
     bzero(&in6_addr, sizeof(in6_addr));
     diag_sprintf(in6_ip, "%s::%s", prefix, inet_ntoa(bp->bp_yiaddr));
@@ -560,7 +631,7 @@ init_net_IPv6(const char *intf, struct bootp *bp, char *prefix)
     in6_addr.ifra_addr.sin6_family = AF_INET6;
     if (!inet_pton(AF_INET6, in6_ip, (char *)&in6_addr.ifra_addr.sin6_addr)) {
         diag_printf("Can't set IPv6 address: %s\n", in6_ip);
-        return false;
+        goto out;
     }
     in6_addr.ifra_prefixmask.sin6_len = sizeof(struct sockaddr_in6);
     in6_addr.ifra_prefixmask.sin6_family = AF_INET6;
@@ -570,10 +641,13 @@ init_net_IPv6(const char *intf, struct bootp *bp, char *prefix)
     in6_addr.ifra_lifetime.ia6t_pltime = ND6_INFINITE_LIFETIME;
     if (ioctl(s, SIOCAIFADDR_IN6, &in6_addr)) {
         perror("SIOCAIFADDR_IN6");
-        return false;
+        goto out;
     }
-    close(s);
-    return true;
+    retcode = true;
+ out:
+    if (s != -1) 
+      close(s);
+    return retcode;
 }
 #endif
 
